#include "Arduino.h"
#include "config.h"
#include "def.h"
#include "EQ.h"
#include "Protocol.h"
#include "MultiWii.h"

// #include "Stepper28.h"

#include "ULN2003.h"

#define  STEPS_DELAY    400
#define  FOCUS_DELAY    800
#define  LON_FIX        0
#define  LAT_FIX        0
#define  TINY_LON_FIX   1
#define  TINY_LAT_FIX   1

#define SHUTTER_TIME_MS  75

// LAT
ULN2003 theStepper(9, 10, 11, 12, STEP_DELAY, 0); // pins for IN1, IN2, IN3, IN4, step delay(in micros)
// LON
ULN2003 theStepper2(7, 6, 5, 4, STEP_DELAY, 0);

static uint8_t nTrace = 1;

uint8_t nPhotoing = 0;                  // Is pressing the shutter.
unsigned long nPhotoingStartTime = 0;   // Time when press the shutter.

uint16_t nShutterIntervalMS = 0;        // Photo interval time.
uint16_t nShutterTimes = 0;             // Number of photos remaining.
unsigned long nShutterTimeMS = 0;       // Estimated photo time.

void MyDebug()
{
  LaserOn();
  delay(200);
  LaserOff();
  delay(200);
  LaserOn();
  delay(200);
  LaserOff();
  delay(200);
  LaserOn();
  delay(200);  
}


void AutoTrace(unsigned long tnStep)
{
  if (nTrace)
  {
    // LonStep(1, tnStep);
    LonStep(0, tnStep);
  }
}

void TraceOn()
{
  nTrace = 1;
}

void TraceOff()
{
  nTrace = 0;
}

void LaserInit()
{
  pinMode(13, OUTPUT);
}

void LaserOn()
{
  digitalWrite(13, HIGH);
}

void LaserOff()
{
  digitalWrite(13, LOW);
}

void FocusInit()
{
}

void LatInit()
{
}

void LonInit()
{
}

void PhotoInit()
{
  pinMode(PIN_PHOTO, INPUT);
  /*
    pinMode(PIN_PHOTO, OUTPUT);
    digitalWrite(PIN_PHOTO, LOW);   // Release
    // */
}

void EQInit()
{
  LaserInit();
  FocusInit();
  LatInit();
  LonInit();

  PhotoInit();
}

void PhotoLoop()
{
  int buttonState = digitalRead(PIN_PHOTO);

  if (buttonState == 0)
  {
    MyDebug();
    LatRev();
    delay(1000);
  }

  /*
    if (nPhotoing)    // Is pressing the shutter.
    {
      unsigned long nNow;

      nNow = millis();
      if (nNow - nPhotoingStartTime > SHUTTER_TIME_MS)  // Time to press the shutter.
      {
        digitalWrite(PIN_PHOTO, LOW);   // Release
        nPhotoing = 0;

        return;
      }
      else
      {
        // Keep waiting, do nothing.
        return;
      }
    }

    if (nShutterTimes > 0)
    {
      unsigned long nNow;
      nNow = millis();
      if (nNow - nShutterTimeMS > nShutterIntervalMS)
      {
        digitalWrite(PIN_PHOTO, HIGH);
        nPhotoingStartTime = nNow;
        nPhotoing = 1;
        nShutterTimes --;
        nShutterTimeMS = nNow;
      }
      else
      {
        // Wait interval time.
        return;
      }
    }
    // */
}

void AutoPhoto(uint16_t tnP1, uint16_t tnP2, uint16_t tnP3)  // 间隔秒数, 触发次数, B门时间
{
  nShutterIntervalMS = tnP1 * 1000;
  nShutterTimes = tnP2;
}

void Photo()
{
  nShutterIntervalMS = 0;
  nShutterTimes = 1;
  nShutterTimeMS = millis();
}

#define PWM1 3
#define PWM2 A0
#define PWM3 A1
#define PWM4 A2

#define FOCUS_STEP_MS  5

void FocusStop()
{
  return;
}

boolean OldLatDir = 0;

void LatStep(boolean tlDir, long tnSteps)
{
  if (tlDir != OldLatDir)
  {
    if (tlDir)
    {
      theStepper.move(tnSteps);
      theStepper.move(LAT_FIX);
    }
    else
    {
      theStepper.move(-tnSteps);
      theStepper.move(-LAT_FIX);
    }
    OldLatDir = tlDir;
  }
  else
  {
    if (tlDir)
    {
      if (tnSteps >= TINY_LAT_FIX)
      {
        theStepper.move(tnSteps);
      }
      else
      {
        theStepper.move(-TINY_LAT_FIX + tnSteps);
        theStepper.move(TINY_LAT_FIX);
      }
    }
    else
    {
      if (tnSteps >= TINY_LAT_FIX)
      {
        theStepper.move(-tnSteps);
      }
      else
      {
        theStepper.move(TINY_LAT_FIX - tnSteps);
        theStepper.move(-TINY_LAT_FIX);
      }
    }
  }
}

boolean OldLonDir = 0;

void LonStep(boolean tlDir, long tnSteps)
{
  if (tlDir != OldLonDir)
  {
    if (tlDir)
    {
      theStepper2.move(tnSteps);
      theStepper2.move(LON_FIX);
    }
    else
    {
      theStepper2.move(-tnSteps);
      theStepper2.move(-LON_FIX);
    }
    OldLonDir = tlDir;
  }
  else
  {
    if (tlDir)
    {
      if (tnSteps >= TINY_LON_FIX)
      {
        theStepper2.move(tnSteps);
      }
      else
      {
        theStepper2.move(-TINY_LON_FIX + tnSteps);
        theStepper2.move(TINY_LON_FIX);
      }
    }
    else
    {
      if (tnSteps >= TINY_LON_FIX)
      {
        theStepper2.move(-tnSteps);
      }
      else
      {
        theStepper2.move(TINY_LON_FIX - tnSteps);
        theStepper2.move(-TINY_LON_FIX);
      }
    }
  }
}

void FocusPositive(uint8_t tnStep)
{
  return;
}

void FocusNegative(uint8_t tnStep)
{
  return;
}

void EQLoop()
{
  theStepper.run();
  theStepper2.run();

  return;
}
