#ifndef ULN2003_H
#define ULN2003_H

#define STOP_DELAY_TIME  50000L

#include "Arduino.h"

class ULN2003
{
  public:
    // Constructor. We define pins for In1, In2, In3 and In4 on a ULN2003AN driver
    // plus the number of steps per rev (which is usually 4096) and the delay
    // between steps (900 works well).
    ULN2003(int _p1, int _p2, int _p3, int _p4, unsigned long _stepdelay);

    // Poll the motor and step it if a step is due, must call this as
    // frequently as possible, but at least once per minimum step interval,
    // preferably in your main loop.
    // return true if the motor is at the target position. (if not does it return false???????)
    boolean run();

    // Move function. We define the number of steps (int) and set stepstogo
    // accordingly. Direction is implicit in the sign (>0 is clockwise, <0 is
    // counterclockwise).
    void move(long steps);

    // Return the current step of the motor.
    int currentPosition();

    // Return the number of steps between the current position to the target
    // position. Positive is clockwise. If 0 the motor is not moving.
    int stepsToGo();

    // Sets a new target position that causes the stepper to stop as quickly as
    // possible and pulls pins low.
    void stop();

  private:
    int phase; //the curret phase setp
    int p1; //output pin
    int p2; //output pin
    int p3; //output pin
    int p4; //output pin
    long currentstep; //the current step number, zero indexed.
    long stepstogo;   //the remaining steps to complete the curret movement.
    unsigned long steptime; //the delay time between steps
    unsigned long _lastStepTime; //the last step time in microseconds
    int readyToStop;
    unsigned long toStopTime;
    void nextStep();   //Called if there are stepstogo (!= 0)
    void stepCW();
    void stepCCW();
    void goto1();
    void goto2();
    void goto3();
    void goto4();
    void goto5();
    void goto6();
    void goto7();
    void goto0();
    void powerUp(); //powers pins at current step to get ready for move
};

#endif
